//---------------------------------------------------------------------
// ATtiny26 8MHz
// Serial to NXT

// http://nicotak.com 
// by Takuya Matsubara / NICO Corp.2008
//

// program internal 8MHz
//avrsp -FL11100100

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "softi2cs.h"


void wait_msec(int i);
//void sio_bitwait(unsigned char dat);
void sio_tx(char sio_txdat);
void timer_wait(unsigned char cnt);


#define MYSLA  0x02    // slave addressiUltrasonicj


#ifndef F_CPU
#define   F_CPU   8000000    //CPUNbNg 8MHz
#endif

#define BAUDRATE 9600        //{[[g
#define SIO_TXMASK   (1<<0)          //Mrbg

#if BAUDRATE==2400
	#define PRESCALE  16        //vXP[l
	#define PRESCALECR 5        //vXP[ݒl
#endif
#if BAUDRATE==4800
	#define PRESCALE  8        //vXP[l
	#define PRESCALECR 4        //vXP[ݒl
#endif
#if BAUDRATE==9600
	#define PRESCALE  4        //vXP[l
	#define PRESCALECR 3        //vXP[ݒl
#endif

#define  SIO_MAXCNT  0x100 //^C}ől

#define SIO_BITCNT    ((F_CPU/PRESCALE)/BAUDRATE)
#define SIO_TCNT    (SIO_MAXCNT-SIO_BITCNT)
#define TCNT_100US  (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/100)))
#define TCNT_80US  (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/80)))
#define TCNT_5US  (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/5)))
#define TCNT_2US  (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/2)))
#define TCNT_1US  (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/1)))

#define BUFFMAX 16

char sio_buffcnt=0;
char sio_readcnt=0;
unsigned char sio_buffer[BUFFMAX];
unsigned char sio_rxdat=0;
unsigned char sio_bitcnt = 0;

//*********************
// Pin definition file
//*********************

#define LED_MASK	(1<<6)
#define LED_PORT   PORTB   // |[g
#define LED_DDR    DDRB    // o͐ݒ
#define LED_ON     LED_PORT|=LED_MASK
#define LED_OFF    LED_PORT&=~LED_MASK

#define SIORX_PIN   PINA
#define SIORX_PORT  PORTA
#define SIORX_DDR   DDRA
#define SIORX_BIT   (1<<7)


#define RXINTR_ENABLE	GIMSK|=(1<<PCIE1)	// 荞݋
#define RXINTR_DISABLE	GIMSK&=~(1<<PCIE1)	// 荞݋֎~



//----------init
void sio_init(void)
{
	wait_msec(1000);	//1000msec

//init intr.
	PORTA = 0xff & ~SIORX_BIT;		//Pullup
	PORTB = 0xff & ~LED_MASK;		//Pullup

	ACSR |= (1<<ACD);	//Disable Analog Comparator 
	GIFR |= (1<<PCIF);	// clear Pin change flag 
 	RXINTR_ENABLE;	// RX荞݋

	sei();
}


//---------- intrrupt
// RX荞
SIGNAL (SIG_PIN_CHANGE)
{
// Routine entered at falling edge
	if(SIORX_PIN & SIORX_BIT)
		return;	// オȂsv

    TCNT1 = SIO_MAXCNT-(SIO_BITCNT/2);   // ^C}̏lݒ   
    TIFR |= (1 << TOV1);    // ^C}I[o[t[tONA
    TIMSK |= (1 << TOIE1);  // ^C}I[o[t[荞݋

	RXINTR_DISABLE;
	LED_ON;
	sio_bitcnt = 10;
}

//^C}0荞
SIGNAL (SIG_OVERFLOW1)
{
    TCNT1 = SIO_TCNT;   // ^C}0̏lݒ   
    TIFR |= (1 << TOV1);    // ^C}I[o[t[tONA
//    TIMSK |= (1 << TOIE1);  // ^C}I[o[t[荞݋
	sio_bitcnt--;
	if(sio_bitcnt==9){	//start bit
		return;
	}
	if(sio_bitcnt==0){	//stop bit
		sio_buffer[(int)sio_buffcnt++] = sio_rxdat;

		if(sio_buffcnt >= BUFFMAX)
			sio_buffcnt=0;

		TIMSK &= ~(1 << TOIE1);  // ^C}I[o[t[荞disable
		LED_OFF;
		GIFR |= (1<<PCIF);	// clear Pinchange flag 
		RXINTR_ENABLE;
		return;
	}
	sio_rxdat >>= 1;
	if(SIORX_PIN & SIORX_BIT)
    	sio_rxdat |= 0x80;
}

//--------Mobt@
unsigned char sio_isit_rx(void)
{
	if(sio_buffcnt == sio_readcnt) return(0);
	return(1);
}

//----------keyboard get buffer
unsigned char sio_getrxbuff(void)
{
	unsigned char temp = sio_buffer[(int)sio_readcnt++];

	if(sio_readcnt >= BUFFMAX)
		sio_readcnt = 0;

	return(temp);
}


#define SIO_PORT PORTB      //ISPp|[g
#define SIO_PIN  PINB       //ISPp|[g
#define SIO_DDR  DDRB       //ISPp|[go͐ݒ


//-----------------------------------------------------------------------
// EGCg
void timer_wait(unsigned char cnt)
{
    TCNT1 = cnt;
    TIFR |= (1 << TOV1);  // TIFR̃rbgNA

    while(!(TIFR & (1 << TOV1)));
    // TOV1rbg1ɂȂ܂
}

void wait_msec(int i)
{
	char z;

	while(i--){
		for(z=0; z<10; z++){
			timer_wait(TCNT_100US);
		}
	}
}

void sio_bitwait(void)
{
    while(!(TIFR & (1 << TOV1)));
    // TOV1rbg1ɂȂ܂

    TCNT1 = SIO_TCNT;
    TIFR |= (1 << TOV1);  // TIFR̃rbgNA
}

//-----------------------------------------------------------------------
// 1oCgM
// sio_txdatFMf[^0x00-0xff

//#define SIO_MARK	SIO_PORT&=~SIO_TXMASK	// ʐM
//#define SIO_SPACE	SIO_PORT|=SIO_TXMASK	// 0

#define SIO_MARK	SIO_PORT|=SIO_TXMASK	// ʐM
#define SIO_SPACE	SIO_PORT&=~SIO_TXMASK	// 0

void sio_tx(char sio_txdat)
{
    unsigned char mask = 0x01;

    //X^[grbg
    SIO_SPACE;
    TCNT1 = SIO_TCNT;
    TIFR |= (1 << TOV1);  // TIFR̃rbgNA
	sio_bitwait();

    //f[^rbg0-7(ʂ瑗M)
    while(mask != 0){    
        if(sio_txdat & mask)
    	    SIO_MARK;
	    else
        	SIO_SPACE;

        mask <<= 1;
		sio_bitwait();
    }
    //Xgbvrbg
    SIO_MARK;
	sio_bitwait();
	sio_bitwait();
}

//---------------------------
int main(void)
{
	unsigned char slaw;
	unsigned char cmd;
	unsigned char cmd2;
	unsigned char ackdat;
	unsigned char val;

    i2c_init();

	LED_DDR |= LED_MASK;
//	LED_ON;

	// VA|[g
	SIO_DDR |= SIO_TXMASK;
    SIO_MARK;	//ʐM

    TCCR1A= 0;                // ^C}1 [h 
    TCCR1B= PRESCALECR;       // ^C}1 vXP[ݒ

	sio_init();
	LED_OFF;

	//debug
//	while(1)
//	{
//		LED_PORT |= LED_MASK;	//LED on
//		sio_tx('A');
//		LED_PORT &= ~LED_MASK;	//LED off
//		wait_msec(1000);	//1000msec
//	}

	while(1)
	{
        while(1){
            i2c_start();               // }X^̐ڑ҂
            slaw = i2c_read_byte();    // AhXM
            if (slaw == MYSLA) {       // AhXv̏ꍇA
                i2c_write_bit(0,1);    // ACKM
                break;
            } else {                   // AhXsv̏ꍇA
                i2c_write_bit(1,1);    // NACKM
            }
        }
//		LED_ON;

        cmd = i2c_read_byte();         // R}hM
        i2c_write_bit(0,1);            // ACKM

        switch(cmd){
        case 0x41:
            cmd2 = i2c_read_byte();    // f[^Mî܂ܔjj
            i2c_write_bit(0,1);        // ACKM
            break;

        case 0x42:	//NXT1oCgM

            i2c_start();               // ʐMĊJ
            cmd2 = i2c_read_byte();    // AhXM
            i2c_write_bit(0,1);        // ACKM

			if(sio_isit_rx()){	//Mf[^
				val = sio_getrxbuff();	
			}else{
				val = 0;	//Mf[^Ȃ
			}

            i2c_write_byte(val);  // lM
            ackdat = i2c_read_bit(1);	//ack
            i2c_sda_low();
            break;
        case 0x43:	//NXT2oCgMioCiΉj
            i2c_start();               // ʐMĊJ
            cmd2 = i2c_read_byte();    // AhXM
            i2c_write_bit(0,1);        // ACKM

			if(sio_isit_rx()){	//Mf[^
    	        i2c_write_byte(1);  // M
	            ackdat = i2c_read_bit(1);	//ack
    	        i2c_write_byte(sio_getrxbuff());  // M

			}else{	//Mf[^Ȃ
    	        i2c_write_byte(0);  // M
    	        ackdat = i2c_read_bit(1);	//ack
    	        i2c_write_byte(0);  // M
			}

            ackdat = i2c_read_bit(1);	//ack
            i2c_sda_low();

            break;

        default:
            break;
        }
        i2c_stop();
//		LED_OFF;
//		sio_tx(cmd);	//debug
//		sio_tx(val);	//debug
	}
	return(0);
}


